/* SmoothWall helper program - setglan
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.
 *
 * (c) Laurent JEAN-RIGAUD, 2001
 * 
 * 12/11/01 LJR initial release
 *
 * filename: setglan.c */
 
#include "setuid.h"

// gestion du fichier ethernet/settings
#define TITLE "SmoothWall v0.9.9"

#define STRING_SIZE 256

#define ADDRESS 0
#define NETADDRESS 1
#define NETMASK 2
#define DHCP 3
#define NETCHANGE_TOTAL 4

#define CONFIG_ROOT "/var/smoothwall/"
/* #define CONFIG_ROOT "" */

struct keyvalue
{
 	char key[STRING_SIZE];
 	char value[STRING_SIZE];
 	struct keyvalue *next;  
};


/* Sets up the list.  First entry is a dummy one to avoid having to special
 * case empty lists. */
struct keyvalue *initkeyvalues(void)
{
 	struct keyvalue *head = malloc(sizeof(struct keyvalue));
	
	strcpy(head->key, "KEY");
	strcpy(head->value, "VALUE");
	head->next = NULL;
	
	return head;
}

/* Splats all the entries in a list. */
void freekeyvalues(struct keyvalue *head)
{
	struct keyvalue *cur = head->next;
	struct keyvalue *next;
	
	while (cur)
	{
		next = cur->next;
		free(cur);
		cur = next;
	}
}

/* Finds a key and copies the value back.  Would be nice to have a func that
 * just returns a pointer to the value? */
int findkey(struct keyvalue *head, char *key, char *value)
{
	struct keyvalue *cur = head->next;
	
	while (cur)
	{
		if (strcmp(key, cur->key) == 0)
		{
			strcpy(value, cur->value);
			return 1;
		}
		cur = cur->next;
	}
	
	return 0;
}

/* Appends a entry.  Not very efficent because it rescans the list looking
 * for the end.  Maybe fix this later. */
void appendkeyvalue(struct keyvalue *head, char *key, char *value)
{
	struct keyvalue *new = malloc(sizeof(struct keyvalue));
	struct keyvalue *cur = head->next;
	struct keyvalue *tail = head;

	strncpy(new->key, key, STRING_SIZE);
	strncpy(new->value, value, STRING_SIZE);
	new->next = NULL;
	
	while (cur)
	{
		tail = cur;	
		cur = cur->next;
	}
	tail->next = new;
}

/* Otherwrites a key with a new value, or if it dosn't exist, appends it
 * on the end. */
void replacekeyvalue(struct keyvalue *head, char *key, char *value)
{
	struct keyvalue *cur = head->next;
	
	while (cur)
	{
		if (strcmp(cur->key, key) == 0)
		{
			strncpy(cur->value, value, STRING_SIZE);
			return;
		}
		cur = cur->next;
	}
	
	appendkeyvalue(head, key, value);
}

/* Reads from a file into a new list.  Uses appendkeyvalue to add entries.
 * Will bomb out on a error (eg bad format line). */
int readkeyvalues(struct keyvalue *head, char *filename)
{
 	FILE *file;
	char buffer[STRING_SIZE];
	char *temp;
	char *key, *value;
	
	if (!(file = fopen(filename, "r")))
		return 0;
		
	while (fgets(buffer, STRING_SIZE, file))
	{
		temp = buffer;
		while (*temp)
		{
			if (*temp =='\n') *temp = '\0';
			temp++;
		}
		if (!strlen(buffer))
			continue;
		if (!(temp = strchr(buffer, '=')))
		{
			fclose(file);
			return 0;
		}
		*temp = '\0';
		key = buffer; value = temp + 1;
		/* See if string is quoted.  If so, skkip first quote, and
		 * nuke the one at the end. */
		if (value[0] == '\'')
		{
			value++;
			if ((temp = strrchr(value, '\'')))
				*temp = '\0';
			else
			{
				fclose(file);
				return 0;
			}
		}
		if (strlen(key))
			appendkeyvalue(head, key, value);
	}
	
	fclose(file);

	return 1;
}

//// MAIN

int main(void)
{
	FILE *ifacefile = NULL;
	FILE *fwdfile = NULL;
	int count;
	char *locip;
	char *locmask;
	char *enabled;
	char s[STRING_SIZE];
	char *result;
	char command[STRING_SIZE];

	// gestion ethernet/settings
	struct keyvalue *kv = initkeyvalues();
	char temp[STRING_SIZE];
	char oIP[STRING_SIZE];
	char oDEV[STRING_SIZE];
	char oMASK[STRING_SIZE];
	char oNETADR[STRING_SIZE];
	char rIP[STRING_SIZE];
	char rDEV[STRING_SIZE];
	char rMASK[STRING_SIZE];
	int configtype;

	if (!(initsetuid()))
		exit(1);

	// lecture configuration ethernet ORANGE et RED
 	if (!(fwdfile = fopen("/var/smoothwall/ethernet/settings", "r")))
	{
		fprintf(stderr, "Couldn't open ethernet settings file");
		goto EXIT;
	}
	fclose(fwdfile);

	if (!(readkeyvalues(kv, CONFIG_ROOT "ethernet/settings")))
	{
		freekeyvalues(kv);
		return 1;
	}	
	strcpy(temp, "0"); 
	findkey(kv, "CONFIG_TYPE", temp); 
	configtype = atol(temp);
	if (configtype < 0 || configtype > 3) configtype = 0;

	if (configtype == 1 || configtype == 3)
	{
		strcpy(oDEV, ""); findkey(kv, "ORANGE_DEV", oDEV);
		// interfacecheck(kv, "ORANGE")
		strcpy(oIP, ""); findkey(kv, "ORANGE_ADDRESS", oIP);
		strcpy(oMASK, ""); findkey(kv, "ORANGE_NETMASK", oMASK);
		strcpy(oNETADR, ""); findkey(kv, "ORANGE_NETADDRESS", oNETADR);
	}
	if (configtype == 2 || configtype == 3)
	{
		strcpy(rDEV, ""); findkey(kv, "RED_DEV", rDEV);
		// interfacecheck(kv, "RED");
		strcpy(rIP, ""); findkey(kv, "RED_ADDRESS", rIP);
		strcpy(rMASK, ""); findkey(kv, "RED_NETMASK", rMASK);
	}
	// fin lecture config eth

 	if (!(fwdfile = fopen("/var/smoothwall/listgreens/config", "r")))
	{
		fprintf(stderr, "Couldn't open listgreens settings file");
		goto EXIT;
	}

	// initialisation du groupe IPCHAINS glan
	system("/sbin/ipchains -F glan");

	while (fgets(s, STRING_SIZE, fwdfile) != NULL)
	{
		if (s[strlen(s) - 1] == '\n')
		        s[strlen(s) - 1] = '\0';		        	
		result = strtok(s, ",");
		count = 0;
		locip = NULL;
		locmask = NULL;
		enabled = NULL;
		while (result)
		{
			if (count == 0)
				locip = result;
			else if (count == 1)
				locmask = result;
			else
				enabled = result;
			count++;
			result = strtok(NULL, ",");
		}

		if (!(locip && locmask && enabled))
			break;
		
		if (strspn(locip, IP_NUMBERS) != strlen(locip))
		{
			fprintf(stderr, "Bad internal IP: %s\n", locip);
			goto EXIT;
		}
		if (strspn(locmask, IP_NUMBERS) != strlen(locmask))
		{
			fprintf(stderr, "Bad internal mask: %s\n", locmask);
			goto EXIT;
		}	

		if (strcmp(enabled, "on") == 0)
		{
			memset(command, 0, STRING_SIZE);
			// Open Internal lan to talk to orange
			snprintf(command, STRING_SIZE - 1, "/sbin/ipchains -A glan -i %s -s %s/%s -d %s/%s -j ACCEPT", oDEV, locip, locmask, oNETADR, oMASK);
			// fprintf(stderr, "setglan: /sbin/ipchains -A glan -i %s -s %s/%s -d %s/%s -j ACCEPT\n", oDEV, locip, locmask, oNETADR, oMASK);
			system(command);

			// Enable green to red (masq)
			memset(command, 0, STRING_SIZE);
			snprintf(command, STRING_SIZE - 1, "/sbin/ipchains -A glan -s %s/%s -i %s -j MASQ", locip, locmask, rDEV);
			// fprintf(stderr, "setglan: /sbin/ipchains -A glan -s %s/%s -i %s -j MASQ\n", locip, locmask, rDEV);
			system(command);
			
		}
	}
		
EXIT:

	freekeyvalues(kv);

	if (ifacefile)
		fclose(ifacefile);
	if (fwdfile)
		fclose(fwdfile);
	
	return 0;
}
